require "iANT/Utils.pl";

sub LuceneListDB
{
    my @a_lucenedb_param = $O_PARAM->GetKeys('lucene_db_\d+');

    my @a_db_list = ();
    foreach my $lucenedb_param (@a_lucenedb_param)
    {
        push(@a_db_list, $O_PARAM->Get($lucenedb_param));
    }
    my $dbs = $O_CGI->popup_menu(
                                 -name     => "lucenedb",
                                 -values   => \@a_db_list,
                                 -override => 1
                                 );

    print STDOUT $O_CGI->start_form();
    print STDOUT $O_CGI->p("Databases available"), $dbs, $O_CGI->submit(), $O_CGI->end_form();
}

sub LuceneGenericRessources
{
    my $what = shift;

    $O_PARAM->Set('store_dir', $O_PARAM->Get('lucene_index_storage') . $O_PARAM->Get('lucene_index_' . $what));
    $O_PARAM->SetAuthorizedCharacters('[&|>/0-9a-zA-Z:_.@+\#\-\s=,\?\(\)%~\[]');

    if ($O_PARAM->Get('suggestion_code') ne '')
    {
        &AddSuggestion();
    }

    if (   ($O_PARAM->Get('value1') ne '')
        || ($O_PARAM->Get('lucene_query') ne '')
        || ($O_PARAM->Get('docid')        ne '')
        || ($O_PARAM->Get('quicksearch')  ne ''))
    {

        &LucenePerformSearch($what);
    }
    else
    {
        &LucenePrintSearchForm($what);
    }

}

sub LuceneQuickSearchForm
{
    print STDOUT $O_CGI->start_form();
    print STDOUT $O_CGI->p($O_CGI->hidden(-name => 'lucenedb', -value => $O_PARAM->Get('lucenedb'), -override => 1));
    print STDOUT $O_CGI->p("Quick search", $O_CGI->textfield(-name => "quicksearch"), $O_CGI->submit());
    print STDOUT $O_CGI->end_form();
}

sub LucenePrintSearchForm
{

    my $what                    = shift();
    my $o_param_cfg_indexfields = New ParamParser($O_PARAM->Get('lucene_cfg_' . $what));
    my @a_search_fields         = ();
    my @a_print_fields          = ();
    my %H_OPERATORS = (
                       ':'   => 'contains',
                       '!~'  => 'does not match (regexp)',
                       '=~'  => 'matches (regexp)',
                       '=='  => 'equals',
                       'gt'  => 'is greater than',
                       'gte' => 'is greater or equal than',
                       'lt'  => 'is lower than',
                       'lte' => 'is lower or equal than'
                       );

    my %H_JOIN = ('&' => 'AND', '|' => 'OR');

    my $lucene_max_fields = 1;
    $lucene_max_fields = $O_PARAM->Get('lucene_max_search_field') if ($O_PARAM->IsDefined('lucene_max_search_field'));

    foreach my $field_name ($o_param_cfg_indexfields->GetKeys('field:\w+'))
    {

        (my $field_shortname) = ($field_name =~ /^field:(\w+)/);
        my $field_index = $o_param_cfg_indexfields->Get($field_name);

        (my $index_type) = ($field_index =~ /^(\w+)\./);

        push(@a_search_fields, $field_shortname) if ($index_type !~ /unindexed/i);
        push(@a_print_fields,  $field_shortname) if ($index_type !~ /unstored/i);

    }

    print STDOUT $O_CGI->h5("Querying $what database");
    print STDOUT $O_CGI->start_form();
    print STDOUT $O_CGI->p(
                           $O_CGI->hidden(
                                          -name     => 'lucene_cfg_' . $what,
                                          -value    => $O_PARAM->Get('lucene_cfg_' . $what),
                                          -override => 1
                             ),
                           $O_CGI->hidden(-name => 'store_dir', -value => $O_PARAM->Get('store_dir'), -override => 1),
                           $O_CGI->hidden(-name => 'lucenedb',  -value => $O_PARAM->Get('lucenedb'),  -override => 1),
                           );

    print STDOUT $O_CGI->p("Direct access via " . $O_CGI->b($o_param_cfg_indexfields->Get('docid')),
                           $O_CGI->textfield(-name => "docid"),
                           $O_CGI->submit());
    print STDOUT $O_CGI->hr;
    my $form_search_fields = '';

    for (my $i = 1; $i <= $lucene_max_fields; $i++)
    {
        my $field = $O_CGI->popup_menu(
                                       -name     => "field$i",
                                       -values   => \@a_search_fields,
                                       -default  => $o_param_cfg_indexfields->Get('default_search'),
                                       -override => 1
                                       );
        my $operator = $O_CGI->popup_menu(
                                          -name     => "operator$i",
                                          -values   => [keys(%H_OPERATORS)],
                                          -labels   => \%H_OPERATORS,
                                          -default  => ':',
                                          -override => 1
                                          );
        my $value = $O_CGI->textfield(-name => "value$i");

        my $join;
        $join = $O_CGI->popup_menu(
                                   -name     => "join$i",
                                   -values   => [keys(%H_JOIN)],
                                   -labels   => \%H_JOIN,
                                   -default  => '&',
                                   -override => 1
                                   ) if ($i > 1);

        $form_search_fields .=
          $O_CGI->Tr($O_CGI->td($join), $O_CGI->td($field), $O_CGI->td($operator), $O_CGI->td($value)) . "\n";

    }

    print STDOUT $O_CGI->p("Search for"),
      $O_CGI->table($form_search_fields,
                    ($O_CGI->Tr($O_CGI->td(), $O_CGI->td(), $O_CGI->td($O_CGI->reset), $O_CGI->td($O_CGI->submit))));

    my %h_labels = ();
    foreach my $print_field (@a_print_fields)
    {
        $h_labels{$print_field} = $print_field;
        $h_labels{$print_field} = 'Raw Xml' if ($print_field eq 'b64_gz_xml');

    }
    print STDOUT $O_CGI->p("Output"),
      $O_CGI->checkbox_group(
                             -name     => 'output_fields',
                             -values   => \@a_print_fields,
                             -labels   => \%h_labels,
                             -defaults => [$o_param_cfg_indexfields->Get('default_out')],
                             -override => 1
                             );

    print STDOUT $O_CGI->end_form();

    &PrintHelp();
}

sub PrintHelp
{
    print STDOUT "<div><p style=\"font-weight: bold;\">Some tips:</p>
			<ul><li>The more you use <em>contains</em> in your query, the fastest it goes</li>
    			<li>Never use &#42; at the begining of a word, use matches(regexp)</li></ul>
		</div><br/>";

}

sub LucenePerformSearch
{

    my $what = shift;

    my $o_param_cfg_indexfields = New ParamParser($O_PARAM->Get('lucene_cfg_' . $what));
    my $lucene_max_fields       = 1;
    $lucene_max_fields = $O_PARAM->Get('lucene_max_search_field') if ($O_PARAM->IsDefined('lucene_max_search_field'));

    my $cmd =
        $O_PARAM->Get('lucene_searcher')
      . ' --parser_cfg='
      . $O_PARAM->Get('lucene_cfg_' . $what)
      . ' --store_dir='
      . $O_PARAM->Get('store_dir');

    my $query = '';

    if ($O_PARAM->IsDefined('lucene_query') && ($O_PARAM->Get('lucene_query') ne ''))
    {
        $query = $O_PARAM->Get('lucene_query');
    }
    elsif ($O_PARAM->IsDefined('docid') && ($O_PARAM->Get('docid') ne ''))
    {

        my $docid_value = $O_PARAM->Get('docid');

        $query = $o_param_cfg_indexfields->Get('docid') . ':' . $docid_value;
    }
    elsif ($O_PARAM->IsDefined('quicksearch') && ($O_PARAM->Get('quicksearch') ne ''))
    {
        $query = '';
        foreach my $field_name ($o_param_cfg_indexfields->GetKeys('field:\w+'))
        {

            (my $field_shortname) = ($field_name =~ /^field:(\w+)/);
            my $field_index = $o_param_cfg_indexfields->Get($field_name);

            (my $index_type) = ($field_index =~ /^(\w+)\./);

            next if ($index_type =~ /unindexed/i);

            $query .= "$field_shortname:\"" . $O_PARAM->Get('quicksearch') . "\" | ";

        }

        $query =~ s/\|\s*$//;

    }
    else
    {
        my %h_substitute_op = (
                               lt  => '<',
                               gt  => '>',
                               lte => '<=',
                               gte => '>=',
                               );
        for (my $i = 1; $i <= $lucene_max_fields; $i++)
        {
            my $value = $O_PARAM->Get("value$i");

            if ($value ne '')
            {
                $query .= ' ' . $O_PARAM->Get("join$i") . ' ' if ($O_PARAM->Get("join$i") ne '');

                my $quote    = '';
                my $operator = $O_PARAM->Get("operator$i");
                $quote = '"'
                  if (($operator eq 'eq') || ($operator eq 'ne') || (($operator eq ':') && ($value !~ /[?*]/)));
                $quote = '/' if (($operator eq '=~') || ($operator eq '!~'));

                my $space = ' ';
                $space = '' if ($operator =~ /^:/);

                if ($operator eq ':[')
                {
                    $quote = '';
                    $value =~ s/\s+to\s+/ TO /i;
                    $value .= ']';
                }
                $operator = $h_substitute_op{$operator} if ($h_substitute_op{$operator});

                my $case_sensitive = '';
                $case_sensitive = 'i' if (($operator eq '=~') || ($operator eq '!~'));

                $query .=
                  $O_PARAM->Get("field$i") . $space . $operator . $space . $quote . $value . "$quote$case_sensitive ";

            }
        }
    }

    $cmd .= ' --query=\'' . $query . '\'';

    my $output_fields = "id";
    if ($O_PARAM->IsDefined('output_fields'))
    {
        $output_fields = $O_PARAM->Get("output_fields");
        $output_fields =~ s/#/,/g;
    }

    if ($output_fields eq 'b64_gz_xml')
    {
        $cmd .= ' --no_output_field_name ';
    }

    $cmd .= ' --output_fields=' . $output_fields;

    my @a_results = `$cmd`;

    print STDOUT $O_CGI->h5("Results for $what search");
    print STDOUT $O_CGI->b("<pre>DEBUG $cmd</pre>") if ($O_PARAM->IsDefined('debug'));
    if ($#a_results < 0)
    {
        print STDOUT $O_CGI->p("No match");
        return;
    }

    if ($output_fields eq 'b64_gz_xml')
    {

        my $raw_data = join('', @a_results);
        print STDOUT "<pre>" . encode_entities($raw_data) . "</pre>";
    }
    else
    {

        print STDOUT "<table >";
        my $first_field = '';
        my $index = 1;
        foreach my $result_line (@a_results)
        {
            chomp $result_line;
            if ($result_line =~ /(\S+)=\s*(.+)/)
            {
                my %h_wb_function = (peptides => 'PeptideRessources', clusters => 'ClusterRessources');
                my $field_name    = $1;
                my $field_value   = $2;
                
                if ($first_field eq '')
                {
                    $first_field = $field_name;
                }
                elsif( $field_name eq $first_field)
                {
                    print STDOUT "<tr><td colspan=\"2\" class=\"up\">\&nbsp;</td></tr>\n";
                }
               
                if ($field_name eq $o_param_cfg_indexfields->Get('docid'))
                {
                    my $href =
                        "?docid=$field_value"
                      . '&lucenedb='
                      . $O_PARAM->Get('lucenedb')
                      . '&output_fields='
                      . $o_param_cfg_indexfields->Get('default_out');
                    $field_name = '<a href=' . $href . ' >' . $field_name . '</a>';
                }
                print STDOUT "<tr><td class=\"section\">$field_name</td><td>$field_value</td></tr>\n";
                
                
                
            }
        }
        print STDOUT "</table>";
    }

}

1;

